@@ -1,35 +1,49 @@
 #structures.py
-from .exceptions import UncancellableEvent
-from .variables import events, async_events
+import asyncio
+import inspect
+from .exceptions import UncancellableEvent, UnfulfilledRequirement
 
 
 # Wrapper for event handlers
 class EventHandler:
-    def __init__(self, function, event, priority, additional_kwargs={}):
+    def __init__(self, function, event: str, priority: int, event_space, additional_kwargs: dict = {}, execution_requirements: dict = {}, requirement_exceptions: bool = False, is_async: bool = False):
         self.function = function
         self.event = event
         self.priority = priority
         self.additional_kwargs = additional_kwargs
+        self.event_space = event_space
+        self.execution_requirements = execution_requirements
+        self.requirement_exceptions = requirement_exceptions
+        self.is_async = is_async
 
+    @property
+    def origin_module(self):
+        return inspect.getmodule(self.function)
 
-    def register(self, additional_kwargs={}):
-        if self.event not in events:
-            events[self.event] = {}
-        
-        # Check if the priority is already registered.
-        if self.priority not in events[self.event]:
-            events[self.event][self.priority] = []
-
-        events[self.event][self.priority].append(self)
-
+    def register(self, additional_kwargs: dict={}):
+        self.event_space.register_handler(self)        
         self.additional_kwargs = self.additional_kwargs | additional_kwargs
 
     def unregister(self):
-        events[self.event][self.priority].remove(self)
+        self.event_space.unregister_handler(self)        
 
-    def call(self, call, args, kwargs):
+    @property
+    def registered(self):
+        return self.event_space.handler_registered(self)
+
+    def call(self, call, args, kwargs):        
         kwargs = kwargs | self.additional_kwargs
-        return self.function(call, *args, **kwargs)
+        for argument in self.execution_requirements:
+            if argument not in kwargs or self.execution_requirements[argument] != kwargs[argument]:
+                if self.requirement_exceptions:
+                    raise UnfulfilledRequirement(f"Argument {argument} was unmet.")
+                return
+        if self.is_async:
+            return asyncio.run(self.function(event=call, *args, **kwargs))
+        return self.function(event=call, *args, **kwargs)
+
+
+
 
 # Add support for event calls inside objects
 class EventSupport:
@@ -40,6 +54,8 @@ class EventSupport:
         for method in self.__dir__():
             method = self.__getattribute__(method)
             if isinstance(method, EventHandler):
+                if method.registered:
+                    method.unregister()
                 method.register({"self": self})
 
 
@@ -51,6 +67,7 @@ class EventCall:
         self.__cancelled = False
         self.cancellable = cancellable        
         self.response = None
+        self.responses = []
     
     # Cancelling is only one-way. 
     def cancel(self):
